<?php

class UserController extends \BaseController {

	/**
	 * Redirects user to profile of him/herself
	 *
	 * @return Redirect to show profile of self
	 */
	public function index()
	{
		return Redirect::action('UserController@show', array(Auth::user()->id()));
	}

	public function login()
	{
		return View::make('login');
	}

	public function authenticate()
	{
		if (Auth::attempt(array('username' => Input::get('username'), 'password' => Input::get('password'))))
		{
    		return Redirect::to('dashboard');
		}
	}

	public function register()
	{
		return View::make('register');
	}

	public function store()
	{
		# Validation
		$rules = array(
			'username' => 'alpha_num|min:6|required',
			'password' => 'min:6|confirmed|required',
			'email' => 'email|required');
		$validator = Validator::make(Input::all(), $rules);
		if( $validator->fails() ) {
			return Redirect::to('register')->withErrors($validator);
		}

		# Create the user
		$user = new User;
		$user->username = Input::get('username');
		$user->password = Hash::make(Input::get('password'));
		$user->email = Input::get('email');
		$user->save();

		# Return successful create
		return Redirect::to('index')->withMessage("Success!");
	}

	/**
	 * Display the specified profile.
	 *
	 * @param  int  $id
	 * @return View of user
	 */
	public function show($id)
	{
		// Fetch the user
		$user = User::find($id);

		return View::make('profile.show', compact('user'));
	}

	/**
	 * Show the form for editing the user's account.
	 *
	 * @return Response
	 */
	public function edit()
	{
		// Fetch the user based on ID of user
		$user = User::find(Auth::user()->id());

		return View::make('profile.edit', compact('user'));
	}

	/**
	 * Update the specified resource in storage.
	 *
	 * @param  int  $id
	 * @return Response
	 */
	public function update($id)
	{
		$profile = User::find($id);

		$rules = array(
			'username' => 'alpha_num|min:6|unique',
			'password' => 'min:6|confirmed',
			'email' => 'email');

		if( $profile ) {
			$validator = Validator::make(Input::all(), $rules);
			if( $validator->fails() ) {
				return Redirect::action('UserController@edit')->withErrors($validator);
			} 

			# Checking inputs to see if update necessary
			if( Input::get('username') != $profile->username ) {
				$profile->username = Input::get('username');
			}

			if( Hash::check( Input::get('password'), $profile->getAuthPassword() ) ) {
				$profile->password = Hash::make(Input::get('password'));
			}

			if( Input::get('email') != $profile->email ) {
				$profile->email = Input::get('email');
			}

			$profile->save();
		}

		return Redirect::to('UserController@show', $id )->withMessage();
	}

	/**
	 * Remove the specified resource from storage.
	 *
	 * @param  int  $id
	 * @return Response
	 */
	public function destroy($id)
	{
		
	}

}